
//******************************************************************************
// hardware definitions
//******************************************************************************
#include "__Lib_FAT32.h"
// RGB Leds
sfr atomic sbit LED_R                     at LATD.B1;
sfr atomic sbit LED_G                     at LATD.B2;
sfr atomic sbit LED_B                     at LATD.B3;

sfr atomic sbit LED_R_Direction           at TRISD.B1;
sfr atomic sbit LED_G_Direction           at TRISD.B2;
sfr atomic sbit LED_B_Direction           at TRISD.B3;

// MMC Connections
sfr atomic sbit Mmc_Chip_Select           at LATA.B6;
sfr atomic sbit Mmc_Chip_Select_Direction at TRISA.B6;
sfr atomic sbit MMC_Card_Detect           at PORTA.B7;
sfr atomic sbit MMC_Card_Detect_Direction at TRISA.B7;

// MP3 Codec Connections
sfr atomic sbit MP3_CS                    at LATA.B5;
sfr atomic sbit MP3_RST                   at LATG.B12;
sfr atomic sbit BSYNC                     at LATG.B13;
sfr atomic sbit DREQ                      at PORTG.B14;

sfr atomic sbit MP3_CS_Direction          at TRISA.B5;
sfr atomic sbit MP3_RST_Direction         at TRISG.B12;
sfr atomic sbit BSYNC_Direction           at TRISG.B13;
sfr atomic sbit DREQ_Direction            at TRISG.B14;

// nRF24L01+ connections
sfr sbit nRF_CS                           at LATA.B4;
sfr sbit nRF_CE                           at LATB.B14;
sfr sbit nRF_IRQ                          at PORTB.B3;

sfr sbit nRF_CS_Direction                 at TRISA.B4;
sfr sbit nRF_CE_Direction                 at TRISB.B14;
sfr sbit nRF_IRQ_Direction                at TRISB.B3;

// extern functions
void Power_Man_Init();

// global flags
char Fat_Initialized_Flag;
signed short Ext_fhandle;
unsigned long Ext_StartSect;
char Ext_res_initialized;

// external flags
extern char Example_State;
extern char RF_used;

const unsigned long mikromedia_plus_pic32mx7_slide_bmp = 0x0000019C;
const unsigned long mikromedia_pic32_slide_bmp         = 0x00031162;
const unsigned long mikroprog_PIC_slide_bmp            = 0x00062128;
const unsigned long mini_boards_12_bmp                 = 0x000930EE;
const unsigned long mp3_note_bmp                       = 0x000C40B4;
const unsigned long mp3_player_note_bmp                = 0x000C7B94;
const unsigned long pic32_compilers_slide_bmp          = 0x000D2B50;
const unsigned long rgb_bitmap_bmp                     = 0x00103B16;
const unsigned long visual_tft_03_bmp                  = 0x00134ADC;
const unsigned long click_02_bmp                       = 0x00165AA2;
const unsigned long Fusion_bmp                         = 0x00196A68;
const unsigned long game_table_bmp                     = 0x001C7A2E;

/*******************************************************************************
* Function Init_GPIO()
* ------------------------------------------------------------------------------
* Overview: Function Initialize GPIO ports
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void Init_GPIO(){
  // init piezo
  Sound_Init(&PORTF, 13);
  
  // init RGB
  LED_R_Direction = 0;
  LED_G_Direction = 0;
  LED_B_Direction = 0;
  
  LED_R = 0;
  LED_G = 0;
  LED_B = 0;
  
  Mmc_Chip_Select_Direction = 0;
  MMC_Card_Detect_Direction = 1;
  
  // Initiliaze Power management
  Power_Man_Init();
  
  // Set default values for flags
  Fat_Initialized_Flag = 0;
  Ext_res_initialized = 0;
  Ext_fhandle = 0xFF;
  Example_State = 0;
  RF_used = 0;
}

char My_Ext_Data_Buffer[512];
unsigned long currentSector = -1, res_file_size;
/*******************************************************************************
* Function MyTFT_Get_Data(unsigned long offset, unsigned long count, unsigned long *num)
* ------------------------------------------------------------------------------
* Overview: Function Sets Get Data Handler for FAT32 format
* Input: offset in RES file, number of bytes to read, number of read bytes
* Output: Pointer to desired position in buffer
*******************************************************************************/
char* MyTFT_Get_Data(unsigned long fPos, unsigned long bytesToGet, unsigned long *pBytesGot) {
unsigned int  scOffset;

  *pBytesGot = 0;

  if (Ext_fhandle < 0) {
    return 0; // return with nil if handle was bad.
  }

  // We will utilize some of the fat32 implied features
  // (such as built in sector buffer) to reduce ram memory consumpiton.
  // You can't use this approach if this buffer is going to be used
  // from another thread (i.e. using fat32 routines in interrupt...)
  // In that case or if you have plenty of ram memory use separate
  // as large as possible data buffer.
  f32_sector.fSectNum = Ext_StartSect + fPos / 512;
  Mmc_Read_Sector(f32_sector.fSectNum, f32_sector.fSect);

  // 512 bytes sector buffer implied.
  scOffset = fPos % 512;
  if(bytesToGet > 512-scOffset)
    *pBytesGot = 512-scOffset;
  else
    *pBytesGot = bytesToGet;

  return &f32_sector.fSect + scOffset;
}

/*******************************************************************************
* Function Init_FAT()
* ------------------------------------------------------------------------------
* Overview: Function Initialize FAT32 system
* Input: Nothing
* Output: 1 if external resources file successfully opened
*         0 if not successful
*******************************************************************************/
char Init_FAT(){
char FAT_cnt = 0;

  Mmc_Chip_Select_Direction = 0;
  MMC_Card_Detect_Direction = 1;
  if (MMC_Card_Detect == 0){
    if (Fat_Initialized_Flag == 0){
      // Initialize SPI2 module
      SPI1_Init_Advanced(_SPI_MASTER, _SPI_8_BIT, 80, _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_END, _SPI_CLK_IDLE_HIGH, _SPI_ACTIVE_2_IDLE);
      while ((Fat32_Init() != 0) && (FAT_cnt < 5))
        Fat_cnt ++;
      if (FAT_cnt < 5){
        Fat_Initialized_Flag = 1;
        // Initialize SPI2 module
        SPI1_Init_Advanced(_SPI_MASTER, _SPI_8_BIT, 8, _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_END, _SPI_CLK_IDLE_HIGH, _SPI_ACTIVE_2_IDLE);
      }
    }
    if (Ext_res_initialized == 0){
      Ext_fhandle    = -1;
      TFT_Set_Ext_Buffer(MyTFT_Get_Data);
      // Open resource file for read
      Ext_fhandle    = FAT32_Open("Ext_reso.RES", 0x01);

      if (Ext_fhandle != 0xFF)
        Ext_StartSect = FAT32_ClustToSect(fat32_fdesc[Ext_fhandle]._1stClust);
        Ext_res_initialized = 1;
    }
  }
  
  return Ext_res_initialized;
}

void Timer_16us_Disable(){
  T3IE_bit = 0;
}

void Timer_16us_Enable(){
  T3IE_bit = 1;
}