/*******************************************************************************
* RTC Driver - Function Set used for RTC module
* ------------------------------------------------------------------------------
*
*******************************************************************************/

// Time struct with am/pm
typedef struct Time {
  short ampm;
  short seconds;
  short minutes;
  short hours;
  short day;
  short month;
  int year;
} TTime;

TTime MyTime, CurrentTime, OldTime;

// variables
static char RTCC_Flag;

/*******************************************************************************
* Function Set_RTC(TTime *RTCC_Time)
* ------------------------------------------------------------------------------
* Overview: Function Sets RTC Time and calendar
* Input: Time struct
* Output: Nothing
*******************************************************************************/
void Set_RTC(TTime *RTCC_Time){
unsigned long temp_time, temp_date;

  RTCCIF_bit = 0;
  RTCCIE_bit = 0;

  CurrentTime = *RTCC_Time;
  
  if (RTCC_Time->ampm == 1){
    if (RTCC_Time->hours < 12)
      RTCC_Time->hours += 12;
  }
  if (RTCC_Time->ampm == 0){
    if (RTCC_Time->hours == 12)
      RTCC_Time->hours = 0;
  }
  
  temp_time = (unsigned long)Dec2Bcd(RTCC_Time->hours) << 24;
  temp_time += (unsigned long)Dec2BCD(RTCC_Time->minutes) << 16;
  temp_time += (unsigned long)Dec2BCD(RTCC_Time->seconds) << 8;
  
  temp_date = (unsigned long)Dec2Bcd(RTCC_Time->year) << 24;
  temp_date += (unsigned long)Dec2BCD(RTCC_Time->month) << 16;
  temp_date += (unsigned long)Dec2BCD(RTCC_Time->day) << 8;
  
  // RTCC registers unlock procedure
  SYSKEY = 0xAA996655;      // Write first unlock key to SYSKEY
  SYSKEY = 0x556699AA;      // Write second unlock key to SYSKEY

  RTCWREN_bit = 1;          // RTC Value registers can be written to by the user

  ON__RTCCON_bit= 0;        // Turn off the RTCC module

  while(RTCCLKON_bit)       // Wait for clock to be turned on
    ;
  RTCTIME = temp_time;      // Set time
  RTCDATE = temp_date;      // Set date
  while(ALRMSYNC_bit)       // Wait for clock to be turned off
    ;

  RTCALRM = 0xC100;         // Alarm every 1 second
  RTSECSEL_bit = 0;         // Use RTC interrupt

  ON__RTCCON_bit= 1;        // Turn on the RTCC module

  while(!RTCCLKON_bit)      // Wait for clock to be turned on
    ;
    
  RTCCIE_bit = 1;
}

/*******************************************************************************
* Function RTCC_ISR() : should be called from RTCC interrupt
* ------------------------------------------------------------------------------
* Overview: Function increments seconds in Time struct
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void RTCC_ISR(){
unsigned long temp1, temp2;
  
  temp1 = RTCTIME;
  temp2 = RTCDATE;
  
  CurrentTime.seconds = Bcd2Dec(temp1 >>  8) ;
  CurrentTime.minutes = Bcd2Dec(temp1 >> 16);
  CurrentTime.hours   = Bcd2Dec(temp1 >> 24);
  
  if (CurrentTime.hours > 11){
    CurrentTime.ampm = 1;
    if (CurrentTime.hours > 12)
      CurrentTime.hours -= 12;
  }
  else {
    CurrentTime.ampm = 0;
    if (CurrentTime.hours == 0)
      CurrentTime.hours = 12;
  }
  
  CurrentTime.day   = Bcd2Dec(temp2 >>  8);
  CurrentTime.month = Bcd2Dec(temp2 >> 16);
  CurrentTime.year  = Bcd2Dec(temp2 >> 24);
  
  RTCC_Flag = 1;
}

/*******************************************************************************
* Function RTCC_Read(TTime *RTCC_Time)
* ------------------------------------------------------------------------------
* Overview: Function reads RTC Time and calendar
* Input: Time struct
* Output: Nothing
*******************************************************************************/
char RTCC_Read(TTime *RTCC_Time){
  char temp = 0;

  if(RTCC_Flag){
    *RTCC_Time = CurrentTime;
    temp = 1;
    RTCC_Flag = 0;
  }

  return temp;
}

/*******************************************************************************
* Function RTC_Init()
* ------------------------------------------------------------------------------
* Overview: Function initializes RTC module
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void RTC_Init(){
  SOSCEN_bit = 1;                    // Enable secondary oscillator
  Delay_ms(100);                     // FOSCEN bit must be enabled in Config bits
  
  RTCCIP0_bit = 1;                   // Set RTCC interrupt
  RTCCIP1_bit = 0;                   // priority level
  RTCCIP2_bit = 1;                   // to 5

  MyTime.seconds = 0;
  MyTime.minutes = 0;
  MyTime.hours = 1;
  MyTime.day = 1;
  MyTime.month = 1;
  MyTime.year = 13;
  MyTime.ampm = 0;
  Set_RTC(&MyTime);
}