/*******************************************************************************
 * Project name:
     Omnibus example for mikromedia + for PIC32MX7
 * Copyright:
     (c) Mikroelektronika, 2011.
 * Revision History:
     20130731:
       - initial release (JK);
 * Description:
     This project allow user to test various on board features of the
     mikromedia + for PIC32MX7 board.

 * Test configuration:
     MCU:             PIC32MX795F512L
                      http://ww1.microchip.com/downloads/en/DeviceDoc/61156H.pdf
     Dev.Board:       mikromedia + for PIC32MX7 : ac:mikromedia_plus_for_PIC32MX7
                      http://www.mikroe.com/mikromedia/plus/pic32mx7/
     Oscillator:      HS-PLL 80.0000 MHz, 16,000 MHz Crystal
     Ext. Modules:    mikromedia + for PIC32MX7 Shield : ac:mikromedia_plus_for_PIC32MX_Shield
                      http://www.mikroe.com/mikromedia/plus/shields/pic32mx7/
     SW:              mikroC PRO for PIC32
                      http://www.mikroe.com/mikroc/pic32/
                      VisualTFT
                      http://www.mikroe.com/visualtft/

 * NOTES:
     - VisualTFT uses following font for example
       Open sans which can be downloaded from here:
       http://www.fontsquirrel.com/fonts/open-sans

     - Accel example reads accel values of all three axes and displays them as
       sliding graph on TFT.

     - Paint example is great way to test touch panel and touch panel controller
       on the board. Simple draw lines and shapes over TFT. Board had STMPE610
       touch panel controller connected to I2C lines on microcontroller.

     - Tic Tac Toe game demonstration. Play simple game with your friend. User
       choose whether the game will be played on one board on two over RF
       communication. For this nRF24L01 is used

     - MP3 player example demonstrates usage of mp3 codec chip and SD card. Program
       uses FAT32 file system and appropriate library can be downloaded from libstock:
       http://www.libstock.com/projects/view/108/fat32-library
       User can navigate through folders and find desired mp3 or wav file and play it.

       IMPORTANT: FAT32 library supports only 8.3 file name format. So all folders and
         audio files have to be in short file name format.

     - Sensors example reads light sensor and if the light intensity is low it dims
       background light. Also temperature Sensors displays measured temperature

     - RTC example is used to set Real Time Clock and Calendar. Board has back up
       battery and it will keep time when power is down. Also user can ajust time
       by setting simple roller wheel, or synhronize with GMT time by pressing sync
       Button. For this purpose mikromedia + for STM Shield is used and our
       Net Ethernet library which is available on libstock:
       http://www.libstock.com/projects/view/107/network-ethernet-library

     - Slide Show example demonstrates Image scrolling. Images are soterd on SD card
       as resource file generated by VisualTFT. scrolling is done by swiping over
       the images or by pressing up or down arrow.

     - RGB Light example demonstrates usage of RGB LED diode. It displays color
       palete on TFT and where ever user touch the Screen it converts it to RGB (5-6-5)
       and sets RGB diode in the same color.

*******************************************************************************/

#include "mikromedia_PIC32_plus_objects.h"
#include "__Lib_nRF24L01.h"

// external declarations
void Run_Example();
void RTC_Init();
char Init_FAT();
void Init_GPIO();
void RTCC_ISR();

extern unsigned int readPort;
extern char newValue;
char TTT_flag = 0;
extern unsigned int TicTacToe_seconds_cnt;

//******************************************************************************
// ISR
//******************************************************************************

//******************************************************************************
//  Real Time Clock and Calendar Interrupt - set to 1 second int
//  Interrupt level 5   ICS: AUTO
//  Used for real time measurement
//******************************************************************************
void RTCC_Intr() iv IVT_RTCC ilevel 5 ics ICS_AUTO {
  RTCCIF_bit = 0;                    // Clear RTCC Alarm flag
  RTCC_ISR();
}

//******************************************************************************
// Timer interrupt - Timer3 (16us)
// Interrupt Level: 7   ICS: SRS
// Used for nRF
//******************************************************************************
void Timer3Interrupt() iv IVT_TIMER_3 ilevel 7 ics ICS_SRS {
  T3IF_bit         = 0;
//  Enter your code here
  TIMER3_IRQ();
}

//******************************************************************************
// Interrupt On Change - RB3
// Interrupt Level: 6   ICS: auto
// Used for nRF
//******************************************************************************
void CN5_Interrupt() iv IVT_CHANGE_NOTICE ilevel 6 ics ICS_AUTO {
  readPort = PORTB;
  newValue = readPort;
  CNIF_bit = 0;        //clear flag

  nRF24L01_IRQ();
}

//******************************************************************************
// Timer interrupt - Timer4_4 32bit (1s)
// Interrupt Level: 4   ICS: auto
// Used for TicTacToe
//******************************************************************************
void Timer4_5Interrupt() iv IVT_TIMER_5 ilevel 4 ics ICS_AUTO{
  T5IF_bit                 = 0;
//  Enter your code here
  TicTacToe_seconds_cnt ++;
  TTT_flag = 1;
  Net_Ethernet_Intern_UserTimerSec++;
}

//******************************************************************************
// Main function
//******************************************************************************
void main() {
  Start_TP();

  Init_GPIO();
  Init_FAT();
  RTC_Init();
  EnableInterrupts();
  while (1) {
//    DisableInterrupts();
    Check_TP();
//    EnableInterrupts();
    Run_Example();
  }
}
